package com.bwie.auth.service.impl;

import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSONObject;
import com.bwie.auth.feign.LoginFeignController;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.LoginRequest;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
public class AuthServiceImpl implements AuthService {
    @Autowired
    private LoginFeignController feignController;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;

    @Override
    public Result login(LoginRequest loginRequest) {
        String userName = loginRequest.getUserName();
        String userPwd = loginRequest.getUserPwd();
        if (StringUtils.isAllBlank(userName, userPwd)) {
            return Result.error("不存在");
        }
        Result<User> name = feignController.findName(userName);
        User data = name.getData();
        if (data == null) {
            return Result.error("未注册");
        }
        if (!ObjectUtil.equal(data.getUserName(), userName)) {
            return Result.error("用户名错误");
        }
        if (!ObjectUtil.equal(data.getUserPwd(), userPwd)) {
            return Result.error("密码错误");
        }
        HashMap<String, Object> map = new HashMap<>();
        String userKey = UUID.randomUUID().toString().replace("-", "");
        map.put(JwtConstants.USER_KEY,userKey);
        String token = JwtUtils.createToken(map);
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+userKey, JSONObject.toJSONString(data),30, TimeUnit.MINUTES);
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setExpirTime("30MIN");
        return Result.success(jwtResponse,"登录成功");
    }

    @Override
    public User userInfo() {
        String token = request.getHeader("token");
        String userKey = JwtUtils.getUserKey(token);
        String s = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        return JSONObject.parseObject(s,User.class);
    }
}
